<?php
namespace Teknisa\SM\Service;

use Teknisa\SM\Factory\Sm00006001TabelaPrecoServicoFactory;
use Teknisa\Libs\Util\Environment;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Query\AST\Functions\LengthFunction;
use Teknisa\SM\Util\Procedure\FctSincronizarVariacaoPreco;
use Teknisa\Libs\Util\NewCode;

class Sm00006001TabelaPrecoServicoService
{
    
    protected $FactoryTabPrecoServico;
    protected $Environment;
    protected $connection;
    
    public function __construct (Sm00006001TabelaPrecoServicoFactory $FactoryTabPrecoServico, Environment $Environment, EntityManager $entityManager, NewCode $newCode){
        
        $this->FactoryTabPrecoServico  = $FactoryTabPrecoServico;
        $this->Environment  = $Environment;
        $this->connection = $entityManager->getConnection();
        $this->newCode   = $newCode;
        
    }
    
    /**
     * A função "validaServicoTabela" verifica se o serviço 
     * que está sendo relacionado já existe está relacionadoao serviço.
     */
     
    public function validaServicoTabela($servOrca, $precBaseServ, $tipoMedida){
    
        $params = array(
            "CDSERVORCA"  => implode(',', $servOrca),
            "CDPRECBASESERV"       => $precBaseServ,
            "CDTPMEDIDASERV"   => $tipoMedida,
            "NRORG"            => $this->Environment->getNrOrg()
        );
        
        $return = $this->FactoryTabPrecoServico->validaServicoTabela($params);
        
        return $return[0]["VALIDASERV"] != 0;
        
    }
    
     /**
     * A função "executarProcedureFctSincronizarVariacaoPreco" executa a procedure
     * "FCT_SINCRONIZAR_VARIACAO_PRECO".
     */
    
    public function executarProcedureFctSincronizarVariacaoPreco($itPrecBaseServ, $tipo, $nrorg, $operador) {
        $procedure = new FctSincronizarVariacaoPreco($this->connection);
        $procedure->executar($itPrecBaseServ, $tipo, $nrorg, $operador);
    }

    public function salvarCopiaOutraMoeda($cdprecbaseserv, $fatorconversao, $nrmoeda, $nmmoeda, $nrorg, $cdoperador) {
        $this->connection->beginTransaction();
        try {
            foreach ($cdprecbaseserv as $row) {
                $pkcdprecbaseserv = $this->newCode->getCodeByTableName("CDPRECBASESERV", $nrorg, 1, 10, true);
                $nomeTabelaPrec = $row["DSPRECBASESERV"].' - '.$nmmoeda;

                $params = array(
                    "CDPRECBASESERV" => $pkcdprecbaseserv,
                    "DSPRECBASESERV" => $nomeTabelaPrec,
                    "NRMOEDA"        => $nrmoeda,
                    "NRORG"          => $nrorg,
                    "CDOPERINCLUSAO" => $cdoperador
                );
                $this->FactoryTabPrecoServico->insertTabPrecos($params);
                $paramsAux = array(
                    "CDPRECBASESERV" => $row["CDPRECBASESERV"],
                );
                $dataServico = $this->FactoryTabPrecoServico->getDadosServico($paramsAux);
                foreach ($dataServico as $itPrecBaseServ) {
                    if ($fatorconversao > 0) {
                        $calculovrpreco = $itPrecBaseServ["VRPRECO"]*$fatorconversao;
                    } else {
                        $calculovrpreco = abs($itPrecBaseServ["VRPRECO"]/$fatorconversao);
                        $calculovrpreco = round($calculovrpreco, 2);
                    }

                    $pknrseqitprecbserv = $this->newCode->getCodeByTableName("FCT_ITPRECBASESERV", $nrorg, 1, 10, true);
                    $paramsServico = array(
                        "NRSEQITPRECBSERV" => $pknrseqitprecbserv,
                        "CDPRECBASESERV"   => $pkcdprecbaseserv,
                        "DTINIPBSERV"      => $itPrecBaseServ["DTINIPBSERV"],
                        "CDTPMEDIDASERV"   => $itPrecBaseServ["CDTPMEDIDASERV"],
                        "CDSERVORCA"       => $itPrecBaseServ["CDSERVORCA"],
                        "VRPRECO"          => $calculovrpreco,
                        "DTFIMVALPBSERV"   => $itPrecBaseServ["DTFIMVALPBSERV"],
                        "CDFILIAL"         => $itPrecBaseServ["CDFILIAL"],
                        "NRORG"            => $nrorg,
                        "DTINCLUSAO"       => $itPrecBaseServ["DTINCLUSAO"],
                        "DTULTATU"         => $itPrecBaseServ["DTULTATU"],
                        "NRORGINCLUSAO"    => $itPrecBaseServ["NRORGINCLUSAO"],
                        "CDOPERINCLUSAO"   => $cdoperador,
                        "NRORGULTATU"      => $itPrecBaseServ["NRORGULTATU"],
                        "CDOPERULTATU"     => $itPrecBaseServ["CDOPERULTATU"],
                        "IDATIVO"          => $itPrecBaseServ["IDATIVO"],
                    );
                    $this->FactoryTabPrecoServico->insertItPrecBaseServ($paramsServico);

                    $paramsAuxVariacao = array(
                        "NRSEQITPRECBSERV" => $itPrecBaseServ["NRSEQITPRECBSERV"],
                    );
                    $dataVariacao = $this->FactoryTabPrecoServico->getDadosVariacao($paramsAuxVariacao);

                    foreach ($dataVariacao as $variacao) {
                        if ($fatorconversao > 0) {
                            $calculoprecovariavel = $variacao["VRPRECOVARIACAO"]*$fatorconversao;
                        } else {
                            $calculoprecovariavel = abs($variacao["VRPRECOVARIACAO"]/$fatorconversao);
                            $calculoprecovariavel = round($calculoprecovariavel, 2);
                        }
                        $pknrvariacaoprecbaseserv = $this->newCode->getCodeByTableName("FCT_VARIACAOPRECBASESERV", $nrorg, 1, 10, true);
                        $paramsVariacao = array(
                            "NRVARIACAOPRECBASESERV"    => $pknrvariacaoprecbaseserv,
                            "NRSEQITPRECBSERV"          => $pknrseqitprecbserv,
                            "CDTPMEDIDASERV"            => $variacao["CDTPMEDIDASERV"],
                            "DTINIPBSERV"               => $variacao["DTINIPBSERV"],
                            "QTMEDIDAINIVARIACAO"       => $variacao["QTMEDIDAINIVARIACAO"],
                            "QTMEDIDAFIMVARIACAO"       => $variacao["QTMEDIDAFIMVARIACAO"],
                            "VRPRECOVARIACAO"           => $calculoprecovariavel,
                            "CDTPMEDIDAVARIAVEL"        => $variacao["CDTPMEDIDAVARIAVEL"],
                            "IDMULTIPVARIACAO"          => $variacao["IDMULTIPVARIACAO"],
                            "NRORG"                     => $nrorg,
                            "IDATIVO"                   => $variacao["IDATIVO"],
                            "DTINCLUSAO"                => $variacao["DTINCLUSAO"],
                            "DTULTATU"                  => $variacao["DTULTATU"],
                            "QTFRANQUIA"                => $variacao["QTFRANQUIA"],
                            "NRORGINCLUSAO"             => $variacao["NRORGINCLUSAO"],
                            "CDOPERINCLUSAO"            => $cdoperador,
                            "NRORGULTATU"               => $variacao["NRORGULTATU"],
                            "CDOPERULTATU"              => $variacao["CDOPERULTATU"],
                        );
                        $this->FactoryTabPrecoServico->insertVariacao($paramsVariacao);
                    }
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollBack();
            throw $e;
        }
    }

}
?>